/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		main.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	21.08.2001
// letzte nderung:	08.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	XGamesSdk Demo Anwendung  nuB
//          dieses Minimal-Demo zeigt, wie man einen Hintergrund und ein Sprite
//          anzeigt
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include <math.h>

#include "XGameSDK\XDisplay.h"
#include "XGameSDK\XSurface.h"
#include "XGameSDK\XSprite.h"
#include "XGameSDK\XApplication.h"
#include "XGameSDK\XLogging.h"
#include "XGameSDK\XTimer.h"


/////////////////////////////////////////////////////////////////////////////////////
//globale Variablen 
XApplication	g_xaApplication;
XDisplay		g_xdDisplay;
XTimer			g_xtTimer;
long			g_lMode;
bool			g_bRunning;
bool			g_bWantToQuit;
bool			g_bESCKey=false;


/////////////////////////////////////////////////////////////////////////////////////
//Grafikobjekte (ebenfalls global angelegt)
XSurface		g_xsurfBackground;
XSprite			g_xsBalls;

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::initGFX
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		ldt die Grafiken in den Speicher und schaltet sie wenn ntig sichtbar
//			von dem Moment an, wo die Grafiken geladen sind, belegen sie
//			VID/SYS-MEM, es empfiehlt sich daher, Grafiken zu entladen, wenn sie
//			lnger nicht gebraucht werden. (z.B. die MenGFX im Ingame)
/////////////////////////////////////////////////////////////////////////////////////
void initGFX(void)
{
	XLogging::log(" --- Grafiken werden geladen ---\n");

    //Hintergrund in eine Surface laden
	g_xsurfBackground.initialize(&g_xdDisplay,"gfx\\back.bmp");

    //Das Sprite mit dem Namen ball.spr laden
	g_xsBalls.initialize(&g_xdDisplay,"gfx\\balls.spr");
	g_xsBalls.m_bVisible=true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::prepareEnd
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		schmeisst die Grafiken wieder aus dem Speicher, und bereitet so das
//			Ende der Applikation vor
/////////////////////////////////////////////////////////////////////////////////////
void prepareEnd(void)
{
    //Hintergrund entladen
	g_xsurfBackground.uninitialize();

    //Sprite entladen
	g_xsBalls.uninitialize();


	XLogging::log(" --- Grafiken entladen ---\n");
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::recover
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		holt Surfaces zurck, die wird z.b. nach einem Taskwechsel ntig
/////////////////////////////////////////////////////////////////////////////////////
void recover(void)
{
    // Display restaurieren
	g_xdDisplay.restore();

    // Hintergrund restaurieren
	g_xsurfBackground.restore();

    // Sprite restaurieren
	g_xsBalls.restore();
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::mainloop
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		die eigentliche Hauptschleife des Programms
/////////////////////////////////////////////////////////////////////////////////////
void mainloop(void)
{
	int iTime;
    int iCount;

    // Timer updaten und aktuelle Zeit holen
	g_xtTimer.update();
    iTime=g_xtTimer.getTime();

	// Hintergrund blitten
	g_xsurfBackground.blit(&g_xdDisplay);


  //////////////////////////////////////////////////////////////
 //Ein Sprite zeichnen
//
        // X-Position des Sprites bestimmen
        g_xsBalls.m_lX=496;
        // Y-Position des Sprites bestimmen
        g_xsBalls.m_lY=368;
        // Frame des Sprites festlegen
        g_xsBalls.m_lFrame=0;
        //Sprite blitten
        g_xsBalls.blit(&g_xdDisplay);


  //////////////////////////////////////////////////////////////
 // Das gleiche Sprite mehrmals zeichnen
//
    for (iCount=31;iCount>=0;iCount--)
    {
        // X-Position des Sprites bestimmen
        g_xsBalls.m_lX=(int)(496.0+460.0*sin (0.0005*(double)(iTime-iCount*150)));
        // Y-Position des Sprites bestimmen
        g_xsBalls.m_lY=(int)(368.0+330.0*cos (0.0015*(double)(iTime-iCount*150)));
        // Frame des Sprites festlegen
        g_xsBalls.m_lFrame=iCount;
        //Sprite blitten
        g_xsBalls.blit(&g_xdDisplay);
    }

    return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::run
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		diese Funktion wird von der main immer aufgerufen, wenn die Anwendung den
//			Fokus hat, und nichts anderes zu tun ist
/////////////////////////////////////////////////////////////////////////////////////
void run(void)
{
	// Mainloop rufen/Bild aufbauen
	mainloop();

	// Bild auf den Screen flippen
	HRESULT hResult	= g_xdDisplay.flip();

	// Bei verlorenen Surfaces, die Surfaces wiederholen
	if (hResult==DDERR_SURFACELOST)
	{
		recover();
	}
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::wndProc
/////////////////////////////////////////////////////////////////////////////////////
// Verarbeitung der Windows Messages
/////////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK wndProc(HWND hWnd,
						 UINT message,
						 WPARAM wParam,
						 LPARAM lParam)
{
	switch (message)
	{
		case WM_KEYDOWN:
			switch (wParam)
			{
                // bei ESC Programm beenden
				case VK_ESCAPE:
					PostMessage(hWnd,WM_CLOSE,0,0);
					g_bESCKey=true;
					break;
			}
			break;
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			// Bei Fokus-Verlust die Ausfuehrung der Anwendung anhalten
			if ((wParam==SIZE_MAXHIDE) || (wParam==SIZE_MINIMIZED))
			{
				g_bRunning	= false;
			}
			else // sonst Ausfuehrung fortsetzen
			{
				g_bRunning	= true;
			}
			break;
	} // switch

	if (g_bWantToQuit) PostMessage(hWnd,WM_CLOSE,0,0);

	return ::DefWindowProc(hWnd,message,wParam,lParam);
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::initialize
/////////////////////////////////////////////////////////////////////////////////////
//INFO:	initialisiert alles, wird zu Beginn des Programms von der main aufgerufen
/////////////////////////////////////////////////////////////////////////////////////
HRESULT initialize(HINSTANCE	hInstance)
{
	HRESULT	hRes;

   	// Anwendungsfenster erzeugen
	g_xaApplication.initialize(hInstance,"XGamesSDK Sample","XGamesSDK Sample",500);
	
	// Timer initialisieren
	g_xtTimer.setUp();
	
	// Programm auf "luft" schalten
	g_bRunning=true;
	
	// Grafikmodi initialiseren
	// wenn einer nicht geht, wird der nchst niedere probiert, bis einer gefunden wird

	hRes=g_xdDisplay.initialize(1024,768,32,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
	if (hRes==DD_OK)
	{
		XLogging::log("1024x768x32 initialized...\n");
		return hRes;
	}
	XLogging::log("1024x768x32 failed...\n");

	g_xdDisplay.uninitialize();
	
	hRes=g_xdDisplay.initialize(1024,768,16,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
	if (hRes==DD_OK)
	{
		XLogging::log("1024x768x16 initialized...\n");
		return hRes;
	}
	XLogging::log("1024x768x16 failed...\n");
	g_xdDisplay.uninitialize();
	
	hRes=g_xdDisplay.initialize(1024,768,15,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
	if (hRes==DD_OK)
	{
		XLogging::log("1024x768x15 initialized...\n");
		return hRes;
	}
	XLogging::log("1024x768x15 failed...\n");
	XLogging::log("Error!!! This videocard is not supported\n");
	
	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
//INFO: Gegenstck zu initialize, wird am Ende der main aufgerufen
/////////////////////////////////////////////////////////////////////////////////////
HRESULT unintialize(void)
{
	HRESULT	hRes	= DD_OK;

	// DirectX deinitialisieren
	g_xdDisplay.uninitialize();
	
	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::WinMain
/////////////////////////////////////////////////////////////////////////////////////
//INFO:		Einsprungpunkt des Programms
/////////////////////////////////////////////////////////////////////////////////////
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG			msg;
	HRESULT		hRes;

	//Programm will erstmal nicht beenden (!)
	g_bWantToQuit=false;
	
	// Logging anwerfen
	XLogging::setContext(true,true,false,"error.log");
	XLogging::log("\n--- Programmstart ---\n\n");

	// Programm-Startup
	hRes=::initialize(hInstance);

	//keine Grafikmode funzt? Programm will beenden!
	if (hRes!=DD_OK)
	{
		g_bWantToQuit=true;	
	}

	// Grafiken laden
	initGFX();

	while (1)
	{
		if (::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE))
		{
			if (!::GetMessage(&msg,NULL,0,0))
			{ // WM_QUIT bekommen
				XLogging::log("\nProgrammende eingeleitet...\n");
				
				// Grafiken entladen
				prepareEnd();

				// while-Schleife beenden
				break;
			}
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
		else
		{
			if (g_bRunning==true)
			{ // wenn die Anwendung den Fokus hat Programm ausfuehren
				run();

			}
		}
	}

	// Programm-Shutdown
	::unintialize();

	XLogging::log("\n--- Programmende ---\n\n");

	return msg.wParam;
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	main.cpp
/////////////////////////////////////////////////////////////////////////////////////
